import requests
import unicodedata
import pyproj

from django.views.generic.detail import SingleObjectMixin, DetailView
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.views.generic.base import View, RedirectView
from django.core.urlresolvers import reverse
from django.http import HttpResponseBadRequest

from passerelle import utils

from .models import MontpellierSig
from .forms import MontpellierSigForm

prefix_map = {
    'ALL': 'ALLEE',
    'AUTO': 'AUTOROUTE',
    'AV': 'AVENUE',
    'BASS': 'BASSIN',
    'BD': 'BOULEVARD',
    'CAR': 'CARREFOUR',
    'CHE': 'CHAUSSEE',
    'COUR': 'COUR',
    'CRS': 'COURS',
    'DESC': 'DESCENTE',
    'DOM': 'DOMAINE',
    'ENCL': 'ENCLOS',
    'ESP': 'ESPLANADE',
    'ESPA': 'ESPACE',
    'GR': '', # "GR GRAND-RUE JEAN MOULIN"
    'IMP': 'IMPASSE',
    'JARD': 'JARDIN',
    'MAIL': '', # "MAIL LE GRAND MAIL"
    'PARC': 'PARC',
    'PARV': '', # "PARV PARVIS DE LA LEGION D HONNEUR"
    'PAS': 'PASSAGE',
    'PL': 'PLACE',
    'PLAN': 'PLAN',
    'PONT': 'PONT',
    'QUA': 'QUAI',
    'R': 'RUE',
    'RAMB': '', # "RAMB RAMBLA DES CALISSONS"
    'RPT': 'ROND-POINT',
    'RTE': 'ROUTE',
    'SQ': 'SQUARE',
    'TSSE': '', # "TSSE TERRASSE DES ALLEES DU BOIS"
    'TUN': 'TUNNEL',
    'VIAD': 'VIADUC',
    'VOI': 'VOIE',
}

def prefix_cleanup(name):
    name = name.strip()
    for prefix, full in prefix_map.items():
        if name.startswith(prefix + ' '):
            name = (full + name[len(prefix):]).strip()
    return name

def split_street(name):
    name = name.strip()
    for prefix, full in prefix_map.items():
        prefix += ' '
        if prefix in name:
            index, name = name.split(prefix)
            index = index.strip()
            name = prefix_cleanup(prefix + name)
    return index, name


class SigCreateView(CreateView):
    model = MontpellierSig
    form_class = MontpellierSigForm
    template_name = 'passerelle/manage/service_form.html'


class SigUpdateView(UpdateView):
    model = MontpellierSig
    form_class = MontpellierSigForm


class SigDeleteView(DeleteView):
    model = MontpellierSig

    def get_success_url(self):
        return reverse('manage-home')


class SigDetailView(DetailView):
    model = MontpellierSig


class CommunesView(View, SingleObjectMixin):
    model = MontpellierSig

    def get(self, request, *args, **kwargs):
        if 'q' in request.GET:
            result = self.get_object().sig_request('commune/' + request.GET['q'])
        else:
            result = self.get_object().sig_request('commune')
        communes = []
        for i in result:
            attrs = i['attributes']
            communes.append({'id': attrs['code'], 'text': attrs['nom']})
        communes.sort(lambda x, y: cmp(x['id'], y['id']))
        return utils.response_for_json(request, {'data': communes})


class VoiesView(View, SingleObjectMixin):
    model = MontpellierSig

    def get(self, request, *args, **kwargs):
        insee = kwargs['insee']
        result = self.get_object().sig_request('commune/' + insee + '/voie')
        voies = []
        for i in result:
            voie = i['attributes']['nom_voie']
            voies.append({'id': voie, 'text': prefix_cleanup(voie)})

        if 'q' in request.GET and request.GET['q']:
            q = request.GET['q'].upper()
            q = q.replace("'", ' ')
            q = unicodedata.normalize('NFKD', q).encode('ascii', 'ignore')
            voies = [v for v in voies if q in v['text']]

        voies.sort(lambda x, y: cmp(x['id'], y['id']))
        return utils.response_for_json(request, {'data': voies})


class VoiesCommuneView(View, SingleObjectMixin):
    model = MontpellierSig

    def get(self, request, *args, **kwargs):
        insee = kwargs['insee']
        nom_rue = kwargs['nom_rue']
        if 'q' in request.GET:
            result = self.get_object().sig_request('commune/' + insee + '/voie/' + nom_rue + '/numero/' + request.GET['q'])
        else:
            result = self.get_object().sig_request('commune/' + insee + '/voie/' + nom_rue + '/numero')
        voies_communes = []
        for i in result:
            attrs = i['attributes']
            voies_communes.append({'id': '%(numero)s %(nom_voie)s' % attrs,
                                   'text': '%(numero)s %(nom_voie)s' % attrs,
                                   'x': attrs['x'], 'y': attrs['y']})
        voies_communes.sort(lambda x, y: cmp(x['id'], y['id']))
        return utils.response_for_json(request, {'data': voies_communes})


class VoieCommuneView(View, SingleObjectMixin):
    model = MontpellierSig

    def get(self, request, *args, **kwargs):
        result = self.get_object().sig_request('voiecommune/' + kwargs['nom_rue'])
        voies_communes = []
        for i in result:
            attrs = i['attributes']
            voies_communes.append({'id': '%(nom_voie)s' % attrs,
                                   'text': '%(nom_voie)s' % attrs,
                                   'commune': attrs['commune'],
                                   'insee': attrs['code_insee']
                               })
        voies_communes.sort(lambda x, y: cmp(x['insee'], y['insee']))
        return utils.response_for_json(request, {'data': voies_communes})


class AdresseView(View, SingleObjectMixin):
    model = MontpellierSig

    def get(self, request, *args, **kwargs):
        lat = request.GET.get('lat')
        lon = request.GET.get('lon')
        # WGS84: epsg:4326
        wgs84 = pyproj.Proj(init='epsg:4326')
        # Lambert93: epsg:2154
        lambert93 = pyproj.Proj(init='epsg:2154')
        try:
            l_lat, l_lon = pyproj.transform(wgs84, lambert93, lat, lon)
        except TypeError:
            return HttpResponseBadRequest()
        result = self.get_object().sig_request('adresse/%s/%s' %(l_lat, l_lon))
        house_number, road = split_street(result.get('voie'))
        address = {'road': road,
                   'house_number': house_number,
                   'city': result.get('commune'),
                   'neighbourhood': result.get('sousquartier'),
                   'postcode': result.get('codepostal'),
                   'suburb': result.get('quartier'),
                   'country': 'France',
        }
        return utils.response_for_json(request, {'address': address})
